@@ -348,10 +348,8 @@ jabber_bosh_connection_send(PurpleBOSHConnection *conn,
 		 * sent immediately), queue up the data and start a timer to flush
 		 * the buffer.
 		 */
-		if (data) {
-			int len = data ? strlen(data) : 0;
-			purple_circ_buffer_append(conn->pending, data, len);
-		}
+		if (data)
+			purple_circ_buffer_append(conn->pending, data, strlen(data));
 
 		if (purple_debug_is_verbose())
 			purple_debug_misc("jabber", "bosh: %p has %" G_GSIZE_FORMAT " bytes in "
@@ -622,27 +620,34 @@ static void jabber_bosh_connection_boot(PurpleBOSHConnection *conn) {
 	g_string_free(buf, TRUE);
 }
 
+/**
+ * Handle one complete BOSH response. This is a <body> node containing
+ * any number of XMPP stanzas.
+ */
 static void
 http_received_cb(const char *data, int len, PurpleBOSHConnection *conn)
 {
+	xmlnode *node;
+	gchar *message;
+
 	if (conn->failed_connections)
 		/* We've got some data, so reset the number of failed connections */
 		conn->failed_connections = 0;
 
-	if (conn->receive_cb) {
-		xmlnode *node = xmlnode_from_str(data, len);
+	g_return_if_fail(conn->receive_cb);
 
-		purple_debug_info("jabber", "RecvBOSH %s(%d): %s\n",
-		                  conn->ssl ? "(ssl)" : "", len, data);
+	node = xmlnode_from_str(data, len);
 
-		if (node) {
-			conn->receive_cb(conn, node);
-			xmlnode_free(node);
-		} else {
-			purple_debug_warning("jabber", "BOSH: Received invalid XML\n");
-		}
+	message = g_strndup(data, len);
+	purple_debug_info("jabber", "RecvBOSH %s(%d): %s\n",
+	                  conn->ssl ? "(ssl)" : "", len, message);
+	g_free(message);
+
+	if (node) {
+		conn->receive_cb(conn, node);
+		xmlnode_free(node);
 	} else {
-		g_return_if_reached();
+		purple_debug_warning("jabber", "BOSH: Received invalid XML\n");
 	}
 }
 
@@ -751,7 +756,12 @@ void jabber_bosh_connection_connect(PurpleBOSHConnection *bosh) {
 	http_connection_connect(conn);
 }
 
-static void
+/**
+ * @return TRUE if we want to be called again immediately. This happens when
+ *         we parse an HTTP response AND there is more data in read_buf. FALSE
+ *         if we should not be called again unless more data has been read.
+ */
+static gboolean
 jabber_bosh_http_connection_process(PurpleHTTPConnection *conn)
 {
 	const char *cursor;
@@ -776,7 +786,7 @@ jabber_bosh_http_connection_process(PurpleHTTPConnection *conn)
 				 * The packet ends in the middle of the Content-Length line.
 				 * We'll try again later when we have more.
 				 */
-				return;
+				return FALSE;
 
 			len = atoi(content_length + strlen("\r\nContent-Length:"));
 			if (len == 0)
@@ -785,10 +795,10 @@ jabber_bosh_http_connection_process(PurpleHTTPConnection *conn)
 			conn->body_len = len;
 		}
 
-		if (connection && (!end_of_headers || content_length < end_of_headers)) {
+		if (connection && (!end_of_headers || connection < end_of_headers)) {
 			const char *tmp;
 			if (strstr(connection, "\r\n") == NULL)
-				return;
+				return FALSE;
 
 
 			tmp = connection + strlen("\r\nConnection:");
@@ -806,17 +816,17 @@ jabber_bosh_http_connection_process(PurpleHTTPConnection *conn)
 			conn->handled_len = end_of_headers - conn->read_buf->str + 4;
 		} else {
 			conn->handled_len = conn->read_buf->len;
-			return;
+			return FALSE;
 		}
 	}
 
 	/* Have we handled everything in the buffer? */
 	if (conn->handled_len >= conn->read_buf->len)
-		return;
+		return FALSE;
 
 	/* Have we read all that the Content-Length promised us? */
 	if (conn->read_buf->len - conn->handled_len < conn->body_len)
-		return;
+		return FALSE;
 
 	--conn->requests;
 	--conn->bosh->requests;
@@ -824,6 +834,14 @@ jabber_bosh_http_connection_process(PurpleHTTPConnection *conn)
 	http_received_cb(conn->read_buf->str + conn->handled_len, conn->body_len,
 	                 conn->bosh);
 
+	/* Is there another response in the buffer ? */
+	if (conn->read_buf->len > conn->body_len + conn->handled_len) {
+		g_string_erase(conn->read_buf, 0, conn->handled_len + conn->body_len);
+		conn->headers_done = FALSE;
+		conn->handled_len = conn->body_len = 0;
+		return TRUE;
+	}
+
 	/* Connection: Close? */
 	if (conn->close && conn->state == HTTP_CONN_CONNECTED) {
 		if (purple_debug_is_verbose())
@@ -842,6 +860,8 @@ jabber_bosh_http_connection_process(PurpleHTTPConnection *conn)
 	conn->read_buf = NULL;
 	conn->headers_done = FALSE;
 	conn->handled_len = conn->body_len = 0;
+
+	return FALSE;
 }
 
 /*
@@ -852,7 +872,7 @@ static void
 http_connection_read(PurpleHTTPConnection *conn)
 {
 	char buffer[1025];
-	int cnt, count = 0;
+	int cnt;
 
 	if (!conn->read_buf)
 		conn->read_buf = g_string_new(NULL);
@@ -864,7 +884,6 @@ http_connection_read(PurpleHTTPConnection *conn)
 			cnt = read(conn->fd, buffer, sizeof(buffer));
 
 		if (cnt > 0) {
-			count += cnt;
 			g_string_append_len(conn->read_buf, buffer, cnt);
 		}
 	} while (cnt > 0);
@@ -886,8 +905,9 @@ http_connection_read(PurpleHTTPConnection *conn)
 		/* Process what we do have */
 	}
 
-	if (conn->read_buf->len > 0)
-		jabber_bosh_http_connection_process(conn);
+	if (conn->read_buf->len > 0) {
+		while (jabber_bosh_http_connection_process(conn));
+	}
 }
 
 static void
