@@ -480,10 +480,12 @@ jabber_disco_stun_srv_resolve_cb(PurpleSrvResponse *resp, int results, gpointer
 	js->srv_query_data = NULL;
 
 	if (results > 0) {
+		PurpleAccount *account;
 		purple_debug_info("jabber", "looking up IP for %s:%d\n",
 			resp[0].hostname, resp[0].port);
+		account = purple_connection_get_account(js->gc);
 		js->stun_query =
-			purple_dnsquery_a(resp[0].hostname, resp[0].port,
+			purple_dnsquery_a_account(account, resp[0].hostname, resp[0].port,
 				jabber_disco_stun_lookup_cb, js);
 	}
 }
@@ -548,7 +550,9 @@ jabber_disco_server_info_result_cb(JabberStream *js, const char *from,
 		} else if (purple_network_get_stun_ip() == NULL ||
 		    purple_strequal(purple_network_get_stun_ip(), "")) {
 			js->srv_query_data =
-				purple_srv_resolve("stun", "udp", js->user->domain,
+				purple_srv_resolve_account(
+					purple_connection_get_account(js->gc), "stun", "udp",
+					js->user->domain,
 					jabber_disco_stun_srv_resolve_cb, js);
 			/* TODO: add TURN support later... */
 		}
