@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA	02111-1301	USA
  *
  */
 
@@ -233,7 +233,7 @@ nm_rtf_strip_formatting(NMRtfContext *ctx, const char *input)
 	if (status == NMRTF_OK)
 		return g_strdup(ctx->output->str);
 
-	purple_debug_info("novell", "RTF parser failed with error code %d", status);
+	purple_debug_info("novell", "RTF parser failed with error code %d\n", status);
 	return NULL;
 }
 
@@ -315,7 +315,6 @@ get_current_encoding(NMRtfContext *ctx)
 			purple_debug_info("novell", "Unhandled font charset %d\n", font->charset);
 			return "CP1252";
 	}
-	return "CP1252";
 }
 
 
