@@ -654,6 +654,8 @@ silcpurple_close_final(gpointer *context)
 {
 	SilcPurple sg = (SilcPurple)context;
 
+	purple_debug_info("silc", "Finalizing SilcPurple %p\n", sg);
+
 	silc_client_stop(sg->client, NULL, NULL);
 	silc_client_free(sg->client);
 	if (sg->sha1hash)
@@ -678,12 +680,12 @@ silcpurple_close(PurpleConnection *gc)
 	g_return_if_fail(sg != NULL);
 
 	ui_info = purple_core_get_ui_info();
-	
+
 	if(ui_info) {
 		ui_name = g_hash_table_lookup(ui_info, "name");
 		ui_website = g_hash_table_lookup(ui_info, "website");
 	}
-	
+
 	if(!ui_name || !ui_website) {
 		ui_name = "Pidgin";
 		ui_website = PURPLE_WEBSITE;
@@ -714,6 +716,8 @@ silcpurple_close(PurpleConnection *gc)
 #endif /* __SILC_TOOLKIT_VERSION */
 
 	purple_timeout_remove(sg->scheduler);
+
+	purple_debug_info("silc", "Scheduling destruction of SilcPurple %p\n", sg);
 	purple_timeout_add(1, (GSourceFunc)silcpurple_close_final, sg);
 }
 
@@ -1112,7 +1116,7 @@ silcpurple_create_keypair_cancel(PurpleConnection *gc, PurpleRequestFields *fiel
 static void
 silcpurple_create_keypair_cb(PurpleConnection *gc, PurpleRequestFields *fields)
 {
-	SilcPurple sg = gc->proto_data;
+	SilcPurple sg;
 	PurpleRequestField *f;
 	const char *val, *pkfile = NULL, *prfile = NULL;
 	const char *pass1 = NULL, *pass2 = NULL, *un = NULL, *hn = NULL;
@@ -1384,8 +1388,7 @@ silcpurple_send_im_resolved(SilcClient client,
 
 	/* Check for images */
 	if (im->gflags & PURPLE_MESSAGE_IMAGES) {
-		list = silcpurple_image_message(im->message,
-						(SilcUInt32 *)(void *)&im->flags);
+		list = silcpurple_image_message(im->message, &im->flags);
 		if (list) {
 			/* Send one or more MIME message.  If more than one, they
 			   are MIME fragments due to over large message */
@@ -1434,7 +1437,7 @@ silcpurple_send_im(PurpleConnection *gc, const char *who, const char *message,
 	SilcClientConnection conn = sg->conn;
 	SilcDList clients;
 	SilcClientEntry client_entry;
-	SilcUInt32 mflags;
+	SilcMessageFlags mflags;
 	char *msg, *tmp;
 	int ret = 0;
 	gboolean sign = purple_account_get_bool(sg->account, "sign-verify", FALSE);
@@ -1795,7 +1798,7 @@ static PurpleCmdRet silcpurple_cmd_cmode(PurpleConversation *conv,
 	}
 
 	silcargs = g_strjoinv(" ", args);
-	silccmd = g_strconcat(cmd, " ", args ? silcargs : NULL, NULL);
+	silccmd = g_strconcat(cmd, " ", silcargs, NULL);
 	g_free(silcargs);
 	if (!silc_client_command_call(sg->client, sg->conn, silccmd)) {
 		g_free(silccmd);
@@ -1845,7 +1848,7 @@ static PurpleCmdRet silcpurple_cmd_quit(PurpleConversation *conv,
 	GHashTable *ui_info;
 	const char *ui_name = NULL, *ui_website = NULL;
 	char *quit_msg;
-								   
+
 	gc = purple_conversation_get_gc(conv);
 
 	if (gc == NULL)
@@ -1857,12 +1860,12 @@ static PurpleCmdRet silcpurple_cmd_quit(PurpleConversation *conv,
 		return PURPLE_CMD_RET_FAILED;
 
 	ui_info = purple_core_get_ui_info();
-	
+
 	if(ui_info) {
 		ui_name = g_hash_table_lookup(ui_info, "name");
 		ui_website = g_hash_table_lookup(ui_info, "website");
 	}
-	
+
 	if(!ui_name || !ui_website) {
 		ui_name = "Pidgin";
 		ui_website = PURPLE_WEBSITE;
@@ -2116,7 +2119,12 @@ static PurplePluginProtocolInfo prpl_info =
 	sizeof(PurplePluginProtocolInfo),       /* struct_size */
 	NULL,				        /* get_account_text_table */
 	NULL,				        /* initiate_media */
-	NULL                        /* can_do_media */
+	NULL,				        /* get_media_caps */
+	NULL,				        /* get_moods */
+	NULL,				        /* set_public_alias */
+	NULL,				        /* get_public_alias */
+	NULL,				        /* add_buddy_with_invite */
+	NULL				        /* add_buddies_with_invite */
 };
 
 static PurplePluginInfo info =
