@@ -100,9 +100,6 @@ struct _GtkSourceUndoAction
 	guint modified  : 1;
 };
 
-/* INVALID is a pointer to an invalid action */
-#define INVALID ((void *) "IA")
-
 struct _GtkSourceUndoManagerPrivate
 {
 	GtkTextBuffer	*document;
@@ -127,9 +124,7 @@ struct _GtkSourceUndoManagerPrivate
 	guint	 	 modified_undoing_group : 1;
 
 	/* Pointer to the action (in the action list) marked as "modified".
-	 * It is NULL when no action is marked as "modified".
-	 * It is INVALID when the action marked as "modified" has been removed
-	 * from the action list (freeing the list or resizing it) */
+	 * It is NULL when no action is marked as "modified". */
 	GtkSourceUndoAction *modified_action;
 };
 
@@ -706,7 +701,7 @@ gtk_source_undo_manager_free_action_list (GtkSourceUndoManager *um)
 			--um->priv->num_of_groups;
 
 		if (action->modified)
-			um->priv->modified_action = INVALID;
+			um->priv->modified_action = NULL;
 
 		gtk_source_undo_action_free (action);
 
@@ -899,7 +894,7 @@ gtk_source_undo_manager_free_first_n_actions (GtkSourceUndoManager	*um,
 			--um->priv->num_of_groups;
 
 		if (action->modified)
-			um->priv->modified_action = INVALID;
+			um->priv->modified_action = NULL;
 
 		gtk_source_undo_action_free (action);
 
@@ -940,7 +935,7 @@ gtk_source_undo_manager_check_list_size (GtkSourceUndoManager *um)
 				--um->priv->num_of_groups;
 
 			if (undo_action->modified)
-				um->priv->modified_action = INVALID;
+				um->priv->modified_action = NULL;
 
 			gtk_source_undo_action_free (undo_action);
 
@@ -1165,9 +1160,7 @@ gtk_source_undo_manager_modified_changed_handler (GtkTextBuffer        *buffer,
 
 		if (um->priv->modified_action != NULL)
 		{
-			if (um->priv->modified_action != INVALID)
-				um->priv->modified_action->modified = FALSE;
-
+			um->priv->modified_action->modified = FALSE;
 			um->priv->modified_action = NULL;
 		}
 
